#!/usr/bin/python
usage = '''Convert a moinmoin wiki to a trac wiki or multiple trac wikis
usage:

    moin2trac.py <wikiname> <moindir> <tracprojectdir> [<wikimapfile>]

    <wikiname> is the name of the moinmoin wiki to import.
    <moindir> is the original moinmoin directory.
    <tracprojectdir> is the target trac project directory
    <wikimapfile>  A mapping of page name to trac wikis
    
   Optionally you may also include <wikimapfile> which
   is useful if you would like to split your wiki into
   a trac multi-project or if you would prefer to map
   only some of you pages into your wiki.

   The syntax is as follow 
   MoinPageName   <tracprojectdir>

   If a <wikimapfile> is specified then only
   those pages in the map file will be transfered
'''

import sys,os
import re
import sre_constants
import time
import socket
from trac.attachment import Attachment
from trac.util.text import to_unicode
# Work for 0.10.3 and 0.11
try:
    from  trac.scripts.admin import TracAdmin
except:
    from trac.admin.console import TracAdmin

sys.path.insert(0, '/etc/moin')
from MoinMoin.Page import Page
from MoinMoin.logfile import editlog
from MoinMoin import wikiutil
try:
    from MoinMoin.request.request_cli import Request
except:
    from MoinMoin.request import RequestCLI as Request

moin_comment_re = re.compile(r'^#.*\n')
moin_toc_re = re.compile(r'<<TableOfContents\((\d*)\)>>')
moin_image_re = re.compile(r'\{\{attachment:([^\|\}]*\S)\s*(\|([^\|\}]*)(\|([^\|\}]*))?)?\}\}')
moin_drawing_re = re.compile(r'\[\[drawing:([^\|\]]*\S)\s*\]\]')
moin_drawing2_re = re.compile(r'drawing:(\S+)')
moin_attlink_re = re.compile(r'\[\[attachment:([^\|\]]*\S)\s*(\|([^\|\]]+))?\]\]')
moin_extlink_re = re.compile(r'\[\[((https?://|ftp://|mailto:)[^\|\]]*\S)\s*(\|([^\]]+))?\]\]')
moin_wikilink_re = re.compile(r'\[\[(MoinMoin:)?([^\|\]]*\S)\s*(\|([^\|\]]+))?\]\]')

moin_inbetween_re = re.compile(r'<<((PageOutline|Image)\([^\)]*\))>>')

def convert(wikiname, moindir, tracdir = None, mapfile = None):
    pagemap = None
    if mapfile:
        pagemap = {}
        for line in open(mapfile):
            if line[0] == '#': continue
            (page, wikidir) = line.split()
            pagemap[page] = wikidir

    pages = os.listdir(moindir)
    for page in pages:
        page = wikiutil.unquoteWikiname(page)

        wikidir = tracdir
        if pagemap:
            if not pagemap.has_key(page): continue
            wikidir = pagemap[page]

        admin  = TracAdmin()
        admin.env_set (wikidir)
        moinrequest = Request(wikiname, str(page))
        moinpage = Page(moinrequest, page)

        if not moinpage.isStandardPage(): continue

        if page[-17:] == '/MoinEditorBackup': continue

        revisions = moinpage.getRevList()
        revisions.sort()
    
        logfile = editlog.EditLog(moinrequest, rootpagename=page)
        logfile.to_begin()

        for rev in revisions:
            revpage = Page(moinrequest, page, rev=rev)
            pagepath = revpage._text_filename()
            if os.access(pagepath, os.F_OK):
                cmd='wiki import %s %s' % ( page,  pagepath)
                print cmd, "->", wikidir

                author = 'trac'
                mtime = int(time.time())
                ip = '127.0.0.1'

                if logfile.size():
                    wanted_rev = "%08d" % rev
                    for line in logfile:
                        if line.rev != wanted_rev: continue

                        # get edit info
                        kind, info = line.getInterwikiEditorData(moinrequest)
                        mtime = line.ed_time_usecs/1000000
                        author = line._usercache[line.userid].name
                        ip = line.addr
                        break

                f = open(pagepath,'r')
                data = to_unicode(f.read(), 'utf-8')

                # remove moin comments
                old_data = None
                while data != old_data:
                    old_data = data
                    data = moin_comment_re.sub(r'', data)

                # update TableOfContents
                data = moin_toc_re.sub(r'<<PageOutline(\1)>>', data)

                # update inline images
                data = moin_image_re.sub(r'<<Image(\1)>>', data)

                # update inline drawings
                data = moin_drawing_re.sub(r'<<Image(\1.png)>>', data)
                data = moin_drawing2_re.sub(r'<<Image(\1.png)>>', data)

                # update attachment links
                try:
                    data = moin_attlink_re.sub(r'[attachment:"\1" \3]', data)
                except sre_constants.error:
                    data = moin_attlink_re.sub(r'[attachment:"\1"]', data)

                # update external links
                try:
                    data = moin_extlink_re.sub(r'[\1 \4]', data)
                except sre_constants.error:
                    data = moin_extlink_re.sub(r'[\1]', data)

                # update wiki links
                try:
                    data = moin_wikilink_re.sub(r'[wiki:"\2" \4]', data)
                except sre_constants.error:
                    data = moin_wikilink_re.sub(r'[wiki:"\2"]', data)

                # replace the <<>> with [[]]
                data = moin_inbetween_re.sub(r'[[\1]]', data)

                admin.db_update("INSERT INTO wiki(version,name,time,author,ipnr,text) "
                   " SELECT 1+COALESCE(max(version),0),%s,%s,"
                   " %s,%s,%s FROM wiki "
                   " WHERE name=%s",
                   None, (page, mtime, author, ip, data, page))

        # Process attachments
        attdir = moinpage.getPagePath('attachments', check_create=0)
        if os.access(attdir, os.F_OK):
            attachments = os.listdir(attdir)
            for att in attachments:
                fname = wikiutil.url_quote(att)
                attachment = Attachment(admin.env_open(), 'wiki', page)

                stats = os.stat(attdir + '/'+ att)
                size = stats[6]

                author = 'trac'
                mtime = stats[8]
                ip = '127.0.0.1'

                if logfile.size():
                    logfile.to_end()
                    for line in logfile.reverse():
                        if line.action == 'ATTNEW' and line.extra == fname:
                            kind, info = line.getInterwikiEditorData(moinrequest)
                            mtime = line.ed_time_usecs/1000000
                            author = line._usercache[line.userid].name
                            ip = line.addr
                            break

                print "attaching " + att + ' = ' + str(size)
                attachment.author = author
                attachment.ipnr = ip
                attfile = open (attdir + '/'+ att)
                attachment.insert (att, attfile, size, t=mtime)

if __name__ == '__main__':
    if len(sys.argv) < 3:
    	print usage
	sys.exit()
    args = sys.argv[1:]
    convert(*args)
